from phoenix.evals.legacy.default_audio_templates import (
    EMOTION_AUDIO_BASE_TEMPLATE_PT_1,
    EMOTION_AUDIO_BASE_TEMPLATE_PT_2,
    EMOTION_AUDIO_BASE_TEMPLATE_PT_3,
    EMOTION_AUDIO_EXPLANATION_TEMPLATE_PT_1,
    EMOTION_AUDIO_EXPLANATION_TEMPLATE_PT_3,
    EMOTION_AUDIO_RAILS,
    EMOTION_PROMPT_TEMPLATE,
)

__all__ = [
    "EMOTION_AUDIO_BASE_TEMPLATE_PT_1",
    "EMOTION_AUDIO_BASE_TEMPLATE_PT_2",
    "EMOTION_AUDIO_BASE_TEMPLATE_PT_3",
    "EMOTION_AUDIO_EXPLANATION_TEMPLATE_PT_1",
    "EMOTION_AUDIO_EXPLANATION_TEMPLATE_PT_3",
    "EMOTION_AUDIO_RAILS",
    "EMOTION_PROMPT_TEMPLATE",
]
